#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/

#include <secp256k1.h>
#include <secp256k1_ecdh.h>
#include <secp256k1_extrakeys.h>
#include <secp256k1_recovery.h>
#include <secp256k1_schnorrsig.h>
#include <secp256k1_ellswift.h>
#include <secp256k1_preallocated.h>
#define PY_USE_BUNDLED

/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context *, unsigned char const *)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 312), // secp256k1_context *
/*  2 */ _CFFI_OP(_CFFI_OP_POINTER, 320), // unsigned char const *
/*  3 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  4 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_ecdsa_recoverable_signature *, unsigned char const *, int)
/*  5 */ _CFFI_OP(_CFFI_OP_POINTER, 312), // secp256k1_context const *
/*  6 */ _CFFI_OP(_CFFI_OP_POINTER, 313), // secp256k1_ecdsa_recoverable_signature *
/*  7 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/*  8 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7), // int
/*  9 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 10 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_ecdsa_recoverable_signature *, unsigned char const *, unsigned char const *, int(*)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *, unsigned int), void const *)
/* 11 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 12 */ _CFFI_OP(_CFFI_OP_NOOP, 6),
/* 13 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 14 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 15 */ _CFFI_OP(_CFFI_OP_POINTER, 254), // int(*)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *, unsigned int)
/* 16 */ _CFFI_OP(_CFFI_OP_POINTER, 329), // void const *
/* 17 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 18 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_ecdsa_signature *, secp256k1_ecdsa_recoverable_signature const *)
/* 19 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 20 */ _CFFI_OP(_CFFI_OP_POINTER, 314), // secp256k1_ecdsa_signature *
/* 21 */ _CFFI_OP(_CFFI_OP_POINTER, 313), // secp256k1_ecdsa_recoverable_signature const *
/* 22 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 23 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_ecdsa_signature *, secp256k1_ecdsa_signature const *)
/* 24 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 25 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 26 */ _CFFI_OP(_CFFI_OP_POINTER, 314), // secp256k1_ecdsa_signature const *
/* 27 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 28 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_ecdsa_signature *, unsigned char const *)
/* 29 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 30 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 31 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 32 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 33 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_ecdsa_signature *, unsigned char const *, size_t)
/* 34 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 35 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 36 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 37 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28), // size_t
/* 38 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 39 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_ecdsa_signature *, unsigned char const *, unsigned char const *, int(*)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *, unsigned int), void const *)
/* 40 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 41 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 42 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 43 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 44 */ _CFFI_OP(_CFFI_OP_NOOP, 15),
/* 45 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 46 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 47 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_ecdsa_signature const *, unsigned char const *, secp256k1_pubkey const *)
/* 48 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 49 */ _CFFI_OP(_CFFI_OP_NOOP, 26),
/* 50 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 51 */ _CFFI_OP(_CFFI_OP_POINTER, 316), // secp256k1_pubkey const *
/* 52 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 53 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_keypair *, unsigned char const *)
/* 54 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 55 */ _CFFI_OP(_CFFI_OP_POINTER, 315), // secp256k1_keypair *
/* 56 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 57 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 58 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_pubkey *)
/* 59 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 60 */ _CFFI_OP(_CFFI_OP_POINTER, 316), // secp256k1_pubkey *
/* 61 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 62 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_pubkey *, secp256k1_ecdsa_recoverable_signature const *, unsigned char const *)
/* 63 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 64 */ _CFFI_OP(_CFFI_OP_NOOP, 60),
/* 65 */ _CFFI_OP(_CFFI_OP_NOOP, 21),
/* 66 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 67 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 68 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_pubkey *, secp256k1_keypair const *)
/* 69 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 70 */ _CFFI_OP(_CFFI_OP_NOOP, 60),
/* 71 */ _CFFI_OP(_CFFI_OP_POINTER, 315), // secp256k1_keypair const *
/* 72 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 73 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_pubkey *, secp256k1_pubkey const * const *, size_t)
/* 74 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 75 */ _CFFI_OP(_CFFI_OP_NOOP, 60),
/* 76 */ _CFFI_OP(_CFFI_OP_POINTER, 51), // secp256k1_pubkey const * const *
/* 77 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 78 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 79 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_pubkey *, secp256k1_xonly_pubkey const *, unsigned char const *)
/* 80 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 81 */ _CFFI_OP(_CFFI_OP_NOOP, 60),
/* 82 */ _CFFI_OP(_CFFI_OP_POINTER, 319), // secp256k1_xonly_pubkey const *
/* 83 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 84 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 85 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_pubkey *, unsigned char const *)
/* 86 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 87 */ _CFFI_OP(_CFFI_OP_NOOP, 60),
/* 88 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 89 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 90 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_pubkey *, unsigned char const *, size_t)
/* 91 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 92 */ _CFFI_OP(_CFFI_OP_NOOP, 60),
/* 93 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 94 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 95 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 96 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_pubkey const *, secp256k1_pubkey const *)
/* 97 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 98 */ _CFFI_OP(_CFFI_OP_NOOP, 51),
/* 99 */ _CFFI_OP(_CFFI_OP_NOOP, 51),
/* 100 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 101 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_xonly_pubkey *, int *, secp256k1_keypair const *)
/* 102 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 103 */ _CFFI_OP(_CFFI_OP_POINTER, 319), // secp256k1_xonly_pubkey *
/* 104 */ _CFFI_OP(_CFFI_OP_POINTER, 8), // int *
/* 105 */ _CFFI_OP(_CFFI_OP_NOOP, 71),
/* 106 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 107 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_xonly_pubkey *, int *, secp256k1_pubkey const *)
/* 108 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 109 */ _CFFI_OP(_CFFI_OP_NOOP, 103),
/* 110 */ _CFFI_OP(_CFFI_OP_NOOP, 104),
/* 111 */ _CFFI_OP(_CFFI_OP_NOOP, 51),
/* 112 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 113 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_xonly_pubkey *, unsigned char const *)
/* 114 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 115 */ _CFFI_OP(_CFFI_OP_NOOP, 103),
/* 116 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 117 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 118 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, secp256k1_xonly_pubkey const *, secp256k1_xonly_pubkey const *)
/* 119 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 120 */ _CFFI_OP(_CFFI_OP_NOOP, 82),
/* 121 */ _CFFI_OP(_CFFI_OP_NOOP, 82),
/* 122 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 123 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char *)
/* 124 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 125 */ _CFFI_OP(_CFFI_OP_POINTER, 320), // unsigned char *
/* 126 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 127 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char *, int *, secp256k1_ecdsa_recoverable_signature const *)
/* 128 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 129 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 130 */ _CFFI_OP(_CFFI_OP_NOOP, 104),
/* 131 */ _CFFI_OP(_CFFI_OP_NOOP, 21),
/* 132 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 133 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char *, secp256k1_ecdsa_signature const *)
/* 134 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 135 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 136 */ _CFFI_OP(_CFFI_OP_NOOP, 26),
/* 137 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 138 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char *, secp256k1_keypair const *)
/* 139 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 140 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 141 */ _CFFI_OP(_CFFI_OP_NOOP, 71),
/* 142 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 143 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char *, secp256k1_pubkey const *, unsigned char const *)
/* 144 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 145 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 146 */ _CFFI_OP(_CFFI_OP_NOOP, 51),
/* 147 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 148 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 149 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char *, secp256k1_pubkey const *, unsigned char const *, int(*)(unsigned char *, unsigned char const *, unsigned char const *, void *), void *)
/* 150 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 151 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 152 */ _CFFI_OP(_CFFI_OP_NOOP, 51),
/* 153 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 154 */ _CFFI_OP(_CFFI_OP_POINTER, 262), // int(*)(unsigned char *, unsigned char const *, unsigned char const *, void *)
/* 155 */ _CFFI_OP(_CFFI_OP_POINTER, 329), // void *
/* 156 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 157 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char *, secp256k1_xonly_pubkey const *)
/* 158 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 159 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 160 */ _CFFI_OP(_CFFI_OP_NOOP, 82),
/* 161 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 162 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char *, size_t *, secp256k1_ecdsa_signature const *)
/* 163 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 164 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 165 */ _CFFI_OP(_CFFI_OP_POINTER, 37), // size_t *
/* 166 */ _CFFI_OP(_CFFI_OP_NOOP, 26),
/* 167 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 168 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char *, size_t *, secp256k1_pubkey const *, unsigned int)
/* 169 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 170 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 171 */ _CFFI_OP(_CFFI_OP_NOOP, 165),
/* 172 */ _CFFI_OP(_CFFI_OP_NOOP, 51),
/* 173 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8), // unsigned int
/* 174 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 175 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char *, unsigned char const *)
/* 176 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 177 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 178 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 179 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 180 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char *, unsigned char const *, secp256k1_keypair const *, unsigned char const *)
/* 181 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 182 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 183 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 184 */ _CFFI_OP(_CFFI_OP_NOOP, 71),
/* 185 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 186 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 187 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char *, unsigned char const *, size_t, secp256k1_keypair const *, secp256k1_schnorrsig_extraparams *)
/* 188 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 189 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 190 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 191 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 192 */ _CFFI_OP(_CFFI_OP_NOOP, 71),
/* 193 */ _CFFI_OP(_CFFI_OP_POINTER, 317), // secp256k1_schnorrsig_extraparams *
/* 194 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 195 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char *, unsigned char const *, size_t, unsigned char const *, size_t)
/* 196 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 197 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 198 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 199 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 200 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 201 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 202 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 203 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char *, unsigned char const *, unsigned char const *)
/* 204 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 205 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 206 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 207 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 208 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 209 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, int, int(*)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *), void *)
/* 210 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 211 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 212 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 213 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 214 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 215 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 216 */ _CFFI_OP(_CFFI_OP_POINTER, 247), // int(*)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *)
/* 217 */ _CFFI_OP(_CFFI_OP_NOOP, 155),
/* 218 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 219 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char const *)
/* 220 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 221 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 222 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 223 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char const *, int, secp256k1_xonly_pubkey const *, unsigned char const *)
/* 224 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 225 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 226 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 227 */ _CFFI_OP(_CFFI_OP_NOOP, 82),
/* 228 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 229 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 230 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(secp256k1_context const *, unsigned char const *, unsigned char const *, size_t, secp256k1_xonly_pubkey const *)
/* 231 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 232 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 233 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 234 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 235 */ _CFFI_OP(_CFFI_OP_NOOP, 82),
/* 236 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 237 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(unsigned char *, unsigned char const *, size_t, unsigned char const *, unsigned char const *, unsigned char const *, size_t, void *)
/* 238 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 239 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 240 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 241 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 242 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 243 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 244 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 245 */ _CFFI_OP(_CFFI_OP_NOOP, 155),
/* 246 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 247 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *)
/* 248 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 249 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 250 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 251 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 252 */ _CFFI_OP(_CFFI_OP_NOOP, 155),
/* 253 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 254 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *, unsigned int)
/* 255 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 256 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 257 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 258 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 259 */ _CFFI_OP(_CFFI_OP_NOOP, 155),
/* 260 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 261 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 262 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // int()(unsigned char *, unsigned char const *, unsigned char const *, void *)
/* 263 */ _CFFI_OP(_CFFI_OP_NOOP, 125),
/* 264 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 265 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 266 */ _CFFI_OP(_CFFI_OP_NOOP, 155),
/* 267 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 268 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // secp256k1_context *()(secp256k1_context const *)
/* 269 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 270 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 271 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // secp256k1_context *()(secp256k1_context const *, void *)
/* 272 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 273 */ _CFFI_OP(_CFFI_OP_NOOP, 155),
/* 274 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 275 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // secp256k1_context *()(unsigned int)
/* 276 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 277 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 278 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // secp256k1_context *()(void *, unsigned int)
/* 279 */ _CFFI_OP(_CFFI_OP_NOOP, 155),
/* 280 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 281 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 282 */ _CFFI_OP(_CFFI_OP_FUNCTION, 306), // secp256k1_scratch_space *()(secp256k1_context const *, size_t)
/* 283 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 284 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 285 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 286 */ _CFFI_OP(_CFFI_OP_FUNCTION, 37), // size_t()(secp256k1_context const *)
/* 287 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 288 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 289 */ _CFFI_OP(_CFFI_OP_FUNCTION, 37), // size_t()(unsigned int)
/* 290 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 291 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 292 */ _CFFI_OP(_CFFI_OP_FUNCTION, 329), // void()(char const *, void *)
/* 293 */ _CFFI_OP(_CFFI_OP_POINTER, 310), // char const *
/* 294 */ _CFFI_OP(_CFFI_OP_NOOP, 155),
/* 295 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 296 */ _CFFI_OP(_CFFI_OP_FUNCTION, 329), // void()(secp256k1_context *)
/* 297 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 298 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 299 */ _CFFI_OP(_CFFI_OP_FUNCTION, 329), // void()(secp256k1_context *, void(*)(char const *, void *), void const *)
/* 300 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 301 */ _CFFI_OP(_CFFI_OP_POINTER, 292), // void(*)(char const *, void *)
/* 302 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 303 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 304 */ _CFFI_OP(_CFFI_OP_FUNCTION, 329), // void()(secp256k1_context const *, secp256k1_scratch_space *)
/* 305 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 306 */ _CFFI_OP(_CFFI_OP_POINTER, 318), // secp256k1_scratch_space *
/* 307 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 308 */ _CFFI_OP(_CFFI_OP_FUNCTION, 329), // void()(void)
/* 309 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 310 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 2), // char
/* 311 */ _CFFI_OP(_CFFI_OP_POINTER, 237), // int(*)(unsigned char *, unsigned char const *, size_t, unsigned char const *, unsigned char const *, unsigned char const *, size_t, void *)
/* 312 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 6), // secp256k1_context
/* 313 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 0), // secp256k1_ecdsa_recoverable_signature
/* 314 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 1), // secp256k1_ecdsa_signature
/* 315 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 2), // secp256k1_keypair
/* 316 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 3), // secp256k1_pubkey
/* 317 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 4), // secp256k1_schnorrsig_extraparams
/* 318 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 7), // secp256k1_scratch_space
/* 319 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 5), // secp256k1_xonly_pubkey
/* 320 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 4), // unsigned char
/* 321 */ _CFFI_OP(_CFFI_OP_ARRAY, 320), // unsigned char[4]
/* 322 */ (_cffi_opcode_t)(4),
/* 323 */ _CFFI_OP(_CFFI_OP_ARRAY, 320), // unsigned char[64]
/* 324 */ (_cffi_opcode_t)(64),
/* 325 */ _CFFI_OP(_CFFI_OP_ARRAY, 320), // unsigned char[65]
/* 326 */ (_cffi_opcode_t)(65),
/* 327 */ _CFFI_OP(_CFFI_OP_ARRAY, 320), // unsigned char[96]
/* 328 */ (_cffi_opcode_t)(96),
/* 329 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_secp256k1_ecdsa_recoverable_signature(secp256k1_ecdsa_recoverable_signature *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { unsigned char(*tmp)[65] = &p->data; (void)tmp; }
}
struct _cffi_align_typedef_secp256k1_ecdsa_recoverable_signature { char x; secp256k1_ecdsa_recoverable_signature y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_secp256k1_ecdsa_signature(secp256k1_ecdsa_signature *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { unsigned char(*tmp)[64] = &p->data; (void)tmp; }
}
struct _cffi_align_typedef_secp256k1_ecdsa_signature { char x; secp256k1_ecdsa_signature y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_secp256k1_keypair(secp256k1_keypair *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { unsigned char(*tmp)[96] = &p->data; (void)tmp; }
}
struct _cffi_align_typedef_secp256k1_keypair { char x; secp256k1_keypair y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_secp256k1_pubkey(secp256k1_pubkey *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { unsigned char(*tmp)[64] = &p->data; (void)tmp; }
}
struct _cffi_align_typedef_secp256k1_pubkey { char x; secp256k1_pubkey y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_secp256k1_schnorrsig_extraparams(secp256k1_schnorrsig_extraparams *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { unsigned char(*tmp)[4] = &p->magic; (void)tmp; }
  { int(* *tmp)(unsigned char *, unsigned char const *, size_t, unsigned char const *, unsigned char const *, unsigned char const *, size_t, void *) = &p->noncefp; (void)tmp; }
  { void * *tmp = &p->ndata; (void)tmp; }
}
struct _cffi_align_typedef_secp256k1_schnorrsig_extraparams { char x; secp256k1_schnorrsig_extraparams y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_secp256k1_xonly_pubkey(secp256k1_xonly_pubkey *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { unsigned char(*tmp)[64] = &p->data; (void)tmp; }
}
struct _cffi_align_typedef_secp256k1_xonly_pubkey { char x; secp256k1_xonly_pubkey y; };

static void _cffi_const_secp256k1_ecdh_hash_function_default(char *o)
{
  *(int(* *)(unsigned char *, unsigned char const *, unsigned char const *, void *))o = secp256k1_ecdh_hash_function_default;
}

static void _cffi_const_secp256k1_ecdh_hash_function_sha256(char *o)
{
  *(int(* *)(unsigned char *, unsigned char const *, unsigned char const *, void *))o = secp256k1_ecdh_hash_function_sha256;
}

static void _cffi_const_secp256k1_ellswift_xdh_hash_function_bip324(char *o)
{
  *(int(* *)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *))o = secp256k1_ellswift_xdh_hash_function_bip324;
}

static void _cffi_const_secp256k1_ellswift_xdh_hash_function_prefix(char *o)
{
  *(int(* *)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *))o = secp256k1_ellswift_xdh_hash_function_prefix;
}

static void _cffi_const_secp256k1_nonce_function_bip340(char *o)
{
  *(int(* *)(unsigned char *, unsigned char const *, size_t, unsigned char const *, unsigned char const *, unsigned char const *, size_t, void *))o = secp256k1_nonce_function_bip340;
}

static void _cffi_const_secp256k1_nonce_function_default(char *o)
{
  *(int(* *)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *, unsigned int))o = secp256k1_nonce_function_default;
}

static void _cffi_const_secp256k1_nonce_function_rfc6979(char *o)
{
  *(int(* *)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *, unsigned int))o = secp256k1_nonce_function_rfc6979;
}

static secp256k1_context * _cffi_d_secp256k1_context_clone(secp256k1_context const * x0)
{
  return secp256k1_context_clone(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_context_clone(PyObject *self, PyObject *arg0)
{
  secp256k1_context const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  secp256k1_context * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_context_clone(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_context_clone _cffi_d_secp256k1_context_clone
#endif

static secp256k1_context * _cffi_d_secp256k1_context_create(unsigned int x0)
{
  return secp256k1_context_create(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_context_create(PyObject *self, PyObject *arg0)
{
  unsigned int x0;
  secp256k1_context * result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, unsigned int);
  if (x0 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_context_create(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_context_create _cffi_d_secp256k1_context_create
#endif

static void _cffi_d_secp256k1_context_destroy(secp256k1_context * x0)
{
  secp256k1_context_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_context_destroy(PyObject *self, PyObject *arg0)
{
  secp256k1_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { secp256k1_context_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_secp256k1_context_destroy _cffi_d_secp256k1_context_destroy
#endif

static secp256k1_context * _cffi_d_secp256k1_context_preallocated_clone(secp256k1_context const * x0, void * x1)
{
  return secp256k1_context_preallocated_clone(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_context_preallocated_clone(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  void * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  secp256k1_context * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "secp256k1_context_preallocated_clone", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(155), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(155), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_context_preallocated_clone(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_context_preallocated_clone _cffi_d_secp256k1_context_preallocated_clone
#endif

static size_t _cffi_d_secp256k1_context_preallocated_clone_size(secp256k1_context const * x0)
{
  return secp256k1_context_preallocated_clone_size(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_context_preallocated_clone_size(PyObject *self, PyObject *arg0)
{
  secp256k1_context const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  size_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_context_preallocated_clone_size(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, size_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_context_preallocated_clone_size _cffi_d_secp256k1_context_preallocated_clone_size
#endif

static secp256k1_context * _cffi_d_secp256k1_context_preallocated_create(void * x0, unsigned int x1)
{
  return secp256k1_context_preallocated_create(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_context_preallocated_create(PyObject *self, PyObject *args)
{
  void * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  secp256k1_context * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "secp256k1_context_preallocated_create", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(155), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(155), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_context_preallocated_create(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_context_preallocated_create _cffi_d_secp256k1_context_preallocated_create
#endif

static void _cffi_d_secp256k1_context_preallocated_destroy(secp256k1_context * x0)
{
  secp256k1_context_preallocated_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_context_preallocated_destroy(PyObject *self, PyObject *arg0)
{
  secp256k1_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { secp256k1_context_preallocated_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_secp256k1_context_preallocated_destroy _cffi_d_secp256k1_context_preallocated_destroy
#endif

static size_t _cffi_d_secp256k1_context_preallocated_size(unsigned int x0)
{
  return secp256k1_context_preallocated_size(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_context_preallocated_size(PyObject *self, PyObject *arg0)
{
  unsigned int x0;
  size_t result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, unsigned int);
  if (x0 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_context_preallocated_size(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, size_t);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_context_preallocated_size _cffi_d_secp256k1_context_preallocated_size
#endif

static int _cffi_d_secp256k1_context_randomize(secp256k1_context * x0, unsigned char const * x1)
{
  return secp256k1_context_randomize(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_context_randomize(PyObject *self, PyObject *args)
{
  secp256k1_context * x0;
  unsigned char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "secp256k1_context_randomize", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_context_randomize(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_context_randomize _cffi_d_secp256k1_context_randomize
#endif

static void _cffi_d_secp256k1_context_set_error_callback(secp256k1_context * x0, void(* x1)(char const *, void *), void const * x2)
{
  secp256k1_context_set_error_callback(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_context_set_error_callback(PyObject *self, PyObject *args)
{
  secp256k1_context * x0;
  void(* x1)(char const *, void *);
  void const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_context_set_error_callback", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (void(*)(char const *, void *))_cffi_to_c_pointer(arg1, _cffi_type(301));
  if (x1 == (void(*)(char const *, void *))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { secp256k1_context_set_error_callback(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_secp256k1_context_set_error_callback _cffi_d_secp256k1_context_set_error_callback
#endif

static void _cffi_d_secp256k1_context_set_illegal_callback(secp256k1_context * x0, void(* x1)(char const *, void *), void const * x2)
{
  secp256k1_context_set_illegal_callback(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_context_set_illegal_callback(PyObject *self, PyObject *args)
{
  secp256k1_context * x0;
  void(* x1)(char const *, void *);
  void const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_context_set_illegal_callback", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (void(*)(char const *, void *))_cffi_to_c_pointer(arg1, _cffi_type(301));
  if (x1 == (void(*)(char const *, void *))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { secp256k1_context_set_illegal_callback(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_secp256k1_context_set_illegal_callback _cffi_d_secp256k1_context_set_illegal_callback
#endif

static int _cffi_d_secp256k1_ec_pubkey_cmp(secp256k1_context const * x0, secp256k1_pubkey const * x1, secp256k1_pubkey const * x2)
{
  return secp256k1_ec_pubkey_cmp(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ec_pubkey_cmp(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_pubkey const * x1;
  secp256k1_pubkey const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_ec_pubkey_cmp", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ec_pubkey_cmp(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ec_pubkey_cmp _cffi_d_secp256k1_ec_pubkey_cmp
#endif

static int _cffi_d_secp256k1_ec_pubkey_combine(secp256k1_context const * x0, secp256k1_pubkey * x1, secp256k1_pubkey const * const * x2, size_t x3)
{
  return secp256k1_ec_pubkey_combine(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ec_pubkey_combine(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_pubkey * x1;
  secp256k1_pubkey const * const * x2;
  size_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "secp256k1_ec_pubkey_combine", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(60), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(60), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(76), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey const * const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(76), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ec_pubkey_combine(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ec_pubkey_combine _cffi_d_secp256k1_ec_pubkey_combine
#endif

static int _cffi_d_secp256k1_ec_pubkey_create(secp256k1_context const * x0, secp256k1_pubkey * x1, unsigned char const * x2)
{
  return secp256k1_ec_pubkey_create(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ec_pubkey_create(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_pubkey * x1;
  unsigned char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_ec_pubkey_create", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(60), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(60), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ec_pubkey_create(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ec_pubkey_create _cffi_d_secp256k1_ec_pubkey_create
#endif

static int _cffi_d_secp256k1_ec_pubkey_negate(secp256k1_context const * x0, secp256k1_pubkey * x1)
{
  return secp256k1_ec_pubkey_negate(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ec_pubkey_negate(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_pubkey * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "secp256k1_ec_pubkey_negate", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(60), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(60), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ec_pubkey_negate(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ec_pubkey_negate _cffi_d_secp256k1_ec_pubkey_negate
#endif

static int _cffi_d_secp256k1_ec_pubkey_parse(secp256k1_context const * x0, secp256k1_pubkey * x1, unsigned char const * x2, size_t x3)
{
  return secp256k1_ec_pubkey_parse(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ec_pubkey_parse(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_pubkey * x1;
  unsigned char const * x2;
  size_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "secp256k1_ec_pubkey_parse", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(60), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(60), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ec_pubkey_parse(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ec_pubkey_parse _cffi_d_secp256k1_ec_pubkey_parse
#endif

static int _cffi_d_secp256k1_ec_pubkey_serialize(secp256k1_context const * x0, unsigned char * x1, size_t * x2, secp256k1_pubkey const * x3, unsigned int x4)
{
  return secp256k1_ec_pubkey_serialize(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ec_pubkey_serialize(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  size_t * x2;
  secp256k1_pubkey const * x3;
  unsigned int x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "secp256k1_ec_pubkey_serialize", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(165), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(165), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x4 = _cffi_to_c_int(arg4, unsigned int);
  if (x4 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ec_pubkey_serialize(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ec_pubkey_serialize _cffi_d_secp256k1_ec_pubkey_serialize
#endif

static int _cffi_d_secp256k1_ec_pubkey_tweak_add(secp256k1_context const * x0, secp256k1_pubkey * x1, unsigned char const * x2)
{
  return secp256k1_ec_pubkey_tweak_add(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ec_pubkey_tweak_add(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_pubkey * x1;
  unsigned char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_ec_pubkey_tweak_add", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(60), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(60), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ec_pubkey_tweak_add(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ec_pubkey_tweak_add _cffi_d_secp256k1_ec_pubkey_tweak_add
#endif

static int _cffi_d_secp256k1_ec_pubkey_tweak_mul(secp256k1_context const * x0, secp256k1_pubkey * x1, unsigned char const * x2)
{
  return secp256k1_ec_pubkey_tweak_mul(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ec_pubkey_tweak_mul(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_pubkey * x1;
  unsigned char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_ec_pubkey_tweak_mul", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(60), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(60), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ec_pubkey_tweak_mul(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ec_pubkey_tweak_mul _cffi_d_secp256k1_ec_pubkey_tweak_mul
#endif

static int _cffi_d_secp256k1_ec_seckey_negate(secp256k1_context const * x0, unsigned char * x1)
{
  return secp256k1_ec_seckey_negate(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ec_seckey_negate(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "secp256k1_ec_seckey_negate", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ec_seckey_negate(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ec_seckey_negate _cffi_d_secp256k1_ec_seckey_negate
#endif

static int _cffi_d_secp256k1_ec_seckey_tweak_add(secp256k1_context const * x0, unsigned char * x1, unsigned char const * x2)
{
  return secp256k1_ec_seckey_tweak_add(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ec_seckey_tweak_add(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  unsigned char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_ec_seckey_tweak_add", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ec_seckey_tweak_add(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ec_seckey_tweak_add _cffi_d_secp256k1_ec_seckey_tweak_add
#endif

static int _cffi_d_secp256k1_ec_seckey_tweak_mul(secp256k1_context const * x0, unsigned char * x1, unsigned char const * x2)
{
  return secp256k1_ec_seckey_tweak_mul(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ec_seckey_tweak_mul(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  unsigned char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_ec_seckey_tweak_mul", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ec_seckey_tweak_mul(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ec_seckey_tweak_mul _cffi_d_secp256k1_ec_seckey_tweak_mul
#endif

static int _cffi_d_secp256k1_ec_seckey_verify(secp256k1_context const * x0, unsigned char const * x1)
{
  return secp256k1_ec_seckey_verify(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ec_seckey_verify(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "secp256k1_ec_seckey_verify", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ec_seckey_verify(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ec_seckey_verify _cffi_d_secp256k1_ec_seckey_verify
#endif

static int _cffi_d_secp256k1_ecdh(secp256k1_context const * x0, unsigned char * x1, secp256k1_pubkey const * x2, unsigned char const * x3, int(* x4)(unsigned char *, unsigned char const *, unsigned char const *, void *), void * x5)
{
  return secp256k1_ecdh(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ecdh(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  secp256k1_pubkey const * x2;
  unsigned char const * x3;
  int(* x4)(unsigned char *, unsigned char const *, unsigned char const *, void *);
  void * x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "secp256k1_ecdh", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x4 = (int(*)(unsigned char *, unsigned char const *, unsigned char const *, void *))_cffi_to_c_pointer(arg4, _cffi_type(154));
  if (x4 == (int(*)(unsigned char *, unsigned char const *, unsigned char const *, void *))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(155), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(155), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ecdh(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ecdh _cffi_d_secp256k1_ecdh
#endif

static int _cffi_d_secp256k1_ecdsa_recover(secp256k1_context const * x0, secp256k1_pubkey * x1, secp256k1_ecdsa_recoverable_signature const * x2, unsigned char const * x3)
{
  return secp256k1_ecdsa_recover(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ecdsa_recover(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_pubkey * x1;
  secp256k1_ecdsa_recoverable_signature const * x2;
  unsigned char const * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "secp256k1_ecdsa_recover", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(60), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(60), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(21), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (secp256k1_ecdsa_recoverable_signature const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(21), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ecdsa_recover(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ecdsa_recover _cffi_d_secp256k1_ecdsa_recover
#endif

static int _cffi_d_secp256k1_ecdsa_recoverable_signature_convert(secp256k1_context const * x0, secp256k1_ecdsa_signature * x1, secp256k1_ecdsa_recoverable_signature const * x2)
{
  return secp256k1_ecdsa_recoverable_signature_convert(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ecdsa_recoverable_signature_convert(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_ecdsa_signature * x1;
  secp256k1_ecdsa_recoverable_signature const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_ecdsa_recoverable_signature_convert", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_ecdsa_signature *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(21), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (secp256k1_ecdsa_recoverable_signature const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(21), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ecdsa_recoverable_signature_convert(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ecdsa_recoverable_signature_convert _cffi_d_secp256k1_ecdsa_recoverable_signature_convert
#endif

static int _cffi_d_secp256k1_ecdsa_recoverable_signature_parse_compact(secp256k1_context const * x0, secp256k1_ecdsa_recoverable_signature * x1, unsigned char const * x2, int x3)
{
  return secp256k1_ecdsa_recoverable_signature_parse_compact(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ecdsa_recoverable_signature_parse_compact(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_ecdsa_recoverable_signature * x1;
  unsigned char const * x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "secp256k1_ecdsa_recoverable_signature_parse_compact", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(6), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_ecdsa_recoverable_signature *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(6), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ecdsa_recoverable_signature_parse_compact(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ecdsa_recoverable_signature_parse_compact _cffi_d_secp256k1_ecdsa_recoverable_signature_parse_compact
#endif

static int _cffi_d_secp256k1_ecdsa_recoverable_signature_serialize_compact(secp256k1_context const * x0, unsigned char * x1, int * x2, secp256k1_ecdsa_recoverable_signature const * x3)
{
  return secp256k1_ecdsa_recoverable_signature_serialize_compact(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ecdsa_recoverable_signature_serialize_compact(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  int * x2;
  secp256k1_ecdsa_recoverable_signature const * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "secp256k1_ecdsa_recoverable_signature_serialize_compact", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(104), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(104), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(21), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (secp256k1_ecdsa_recoverable_signature const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(21), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ecdsa_recoverable_signature_serialize_compact(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ecdsa_recoverable_signature_serialize_compact _cffi_d_secp256k1_ecdsa_recoverable_signature_serialize_compact
#endif

static int _cffi_d_secp256k1_ecdsa_sign(secp256k1_context const * x0, secp256k1_ecdsa_signature * x1, unsigned char const * x2, unsigned char const * x3, int(* x4)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *, unsigned int), void const * x5)
{
  return secp256k1_ecdsa_sign(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ecdsa_sign(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_ecdsa_signature * x1;
  unsigned char const * x2;
  unsigned char const * x3;
  int(* x4)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *, unsigned int);
  void const * x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "secp256k1_ecdsa_sign", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_ecdsa_signature *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x4 = (int(*)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *, unsigned int))_cffi_to_c_pointer(arg4, _cffi_type(15));
  if (x4 == (int(*)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *, unsigned int))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ecdsa_sign(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ecdsa_sign _cffi_d_secp256k1_ecdsa_sign
#endif

static int _cffi_d_secp256k1_ecdsa_sign_recoverable(secp256k1_context const * x0, secp256k1_ecdsa_recoverable_signature * x1, unsigned char const * x2, unsigned char const * x3, int(* x4)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *, unsigned int), void const * x5)
{
  return secp256k1_ecdsa_sign_recoverable(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ecdsa_sign_recoverable(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_ecdsa_recoverable_signature * x1;
  unsigned char const * x2;
  unsigned char const * x3;
  int(* x4)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *, unsigned int);
  void const * x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "secp256k1_ecdsa_sign_recoverable", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(6), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_ecdsa_recoverable_signature *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(6), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x4 = (int(*)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *, unsigned int))_cffi_to_c_pointer(arg4, _cffi_type(15));
  if (x4 == (int(*)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *, unsigned int))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ecdsa_sign_recoverable(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ecdsa_sign_recoverable _cffi_d_secp256k1_ecdsa_sign_recoverable
#endif

static int _cffi_d_secp256k1_ecdsa_signature_normalize(secp256k1_context const * x0, secp256k1_ecdsa_signature * x1, secp256k1_ecdsa_signature const * x2)
{
  return secp256k1_ecdsa_signature_normalize(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ecdsa_signature_normalize(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_ecdsa_signature * x1;
  secp256k1_ecdsa_signature const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_ecdsa_signature_normalize", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_ecdsa_signature *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(26), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (secp256k1_ecdsa_signature const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(26), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ecdsa_signature_normalize(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ecdsa_signature_normalize _cffi_d_secp256k1_ecdsa_signature_normalize
#endif

static int _cffi_d_secp256k1_ecdsa_signature_parse_compact(secp256k1_context const * x0, secp256k1_ecdsa_signature * x1, unsigned char const * x2)
{
  return secp256k1_ecdsa_signature_parse_compact(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ecdsa_signature_parse_compact(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_ecdsa_signature * x1;
  unsigned char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_ecdsa_signature_parse_compact", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_ecdsa_signature *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ecdsa_signature_parse_compact(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ecdsa_signature_parse_compact _cffi_d_secp256k1_ecdsa_signature_parse_compact
#endif

static int _cffi_d_secp256k1_ecdsa_signature_parse_der(secp256k1_context const * x0, secp256k1_ecdsa_signature * x1, unsigned char const * x2, size_t x3)
{
  return secp256k1_ecdsa_signature_parse_der(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ecdsa_signature_parse_der(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_ecdsa_signature * x1;
  unsigned char const * x2;
  size_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "secp256k1_ecdsa_signature_parse_der", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_ecdsa_signature *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ecdsa_signature_parse_der(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ecdsa_signature_parse_der _cffi_d_secp256k1_ecdsa_signature_parse_der
#endif

static int _cffi_d_secp256k1_ecdsa_signature_serialize_compact(secp256k1_context const * x0, unsigned char * x1, secp256k1_ecdsa_signature const * x2)
{
  return secp256k1_ecdsa_signature_serialize_compact(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ecdsa_signature_serialize_compact(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  secp256k1_ecdsa_signature const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_ecdsa_signature_serialize_compact", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(26), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (secp256k1_ecdsa_signature const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(26), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ecdsa_signature_serialize_compact(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ecdsa_signature_serialize_compact _cffi_d_secp256k1_ecdsa_signature_serialize_compact
#endif

static int _cffi_d_secp256k1_ecdsa_signature_serialize_der(secp256k1_context const * x0, unsigned char * x1, size_t * x2, secp256k1_ecdsa_signature const * x3)
{
  return secp256k1_ecdsa_signature_serialize_der(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ecdsa_signature_serialize_der(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  size_t * x2;
  secp256k1_ecdsa_signature const * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "secp256k1_ecdsa_signature_serialize_der", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(165), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(165), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(26), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (secp256k1_ecdsa_signature const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(26), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ecdsa_signature_serialize_der(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ecdsa_signature_serialize_der _cffi_d_secp256k1_ecdsa_signature_serialize_der
#endif

static int _cffi_d_secp256k1_ecdsa_verify(secp256k1_context const * x0, secp256k1_ecdsa_signature const * x1, unsigned char const * x2, secp256k1_pubkey const * x3)
{
  return secp256k1_ecdsa_verify(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ecdsa_verify(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_ecdsa_signature const * x1;
  unsigned char const * x2;
  secp256k1_pubkey const * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "secp256k1_ecdsa_verify", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(26), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_ecdsa_signature const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(26), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ecdsa_verify(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ecdsa_verify _cffi_d_secp256k1_ecdsa_verify
#endif

static int _cffi_d_secp256k1_ellswift_create(secp256k1_context const * x0, unsigned char * x1, unsigned char const * x2, unsigned char const * x3)
{
  return secp256k1_ellswift_create(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ellswift_create(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  unsigned char const * x2;
  unsigned char const * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "secp256k1_ellswift_create", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ellswift_create(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ellswift_create _cffi_d_secp256k1_ellswift_create
#endif

static int _cffi_d_secp256k1_ellswift_decode(secp256k1_context const * x0, secp256k1_pubkey * x1, unsigned char const * x2)
{
  return secp256k1_ellswift_decode(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ellswift_decode(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_pubkey * x1;
  unsigned char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_ellswift_decode", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(60), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(60), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ellswift_decode(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ellswift_decode _cffi_d_secp256k1_ellswift_decode
#endif

static int _cffi_d_secp256k1_ellswift_encode(secp256k1_context const * x0, unsigned char * x1, secp256k1_pubkey const * x2, unsigned char const * x3)
{
  return secp256k1_ellswift_encode(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ellswift_encode(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  secp256k1_pubkey const * x2;
  unsigned char const * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "secp256k1_ellswift_encode", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ellswift_encode(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ellswift_encode _cffi_d_secp256k1_ellswift_encode
#endif

static int _cffi_d_secp256k1_ellswift_xdh(secp256k1_context const * x0, unsigned char * x1, unsigned char const * x2, unsigned char const * x3, unsigned char const * x4, int x5, int(* x6)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *), void * x7)
{
  return secp256k1_ellswift_xdh(x0, x1, x2, x3, x4, x5, x6, x7);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_ellswift_xdh(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  unsigned char const * x2;
  unsigned char const * x3;
  unsigned char const * x4;
  int x5;
  int(* x6)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *);
  void * x7;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;

  if (!PyArg_UnpackTuple(args, "secp256k1_ellswift_xdh", 8, 8, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  x6 = (int(*)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *))_cffi_to_c_pointer(arg6, _cffi_type(216));
  if (x6 == (int(*)(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *, void *))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(155), arg7, (char **)&x7);
  if (datasize != 0) {
    x7 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(155), arg7, (char **)&x7,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_ellswift_xdh(x0, x1, x2, x3, x4, x5, x6, x7); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_ellswift_xdh _cffi_d_secp256k1_ellswift_xdh
#endif

static int _cffi_d_secp256k1_keypair_create(secp256k1_context const * x0, secp256k1_keypair * x1, unsigned char const * x2)
{
  return secp256k1_keypair_create(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_keypair_create(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_keypair * x1;
  unsigned char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_keypair_create", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(55), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_keypair *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(55), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_keypair_create(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_keypair_create _cffi_d_secp256k1_keypair_create
#endif

static int _cffi_d_secp256k1_keypair_pub(secp256k1_context const * x0, secp256k1_pubkey * x1, secp256k1_keypair const * x2)
{
  return secp256k1_keypair_pub(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_keypair_pub(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_pubkey * x1;
  secp256k1_keypair const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_keypair_pub", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(60), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(60), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(71), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (secp256k1_keypair const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(71), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_keypair_pub(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_keypair_pub _cffi_d_secp256k1_keypair_pub
#endif

static int _cffi_d_secp256k1_keypair_sec(secp256k1_context const * x0, unsigned char * x1, secp256k1_keypair const * x2)
{
  return secp256k1_keypair_sec(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_keypair_sec(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  secp256k1_keypair const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_keypair_sec", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(71), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (secp256k1_keypair const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(71), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_keypair_sec(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_keypair_sec _cffi_d_secp256k1_keypair_sec
#endif

static int _cffi_d_secp256k1_keypair_xonly_pub(secp256k1_context const * x0, secp256k1_xonly_pubkey * x1, int * x2, secp256k1_keypair const * x3)
{
  return secp256k1_keypair_xonly_pub(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_keypair_xonly_pub(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_xonly_pubkey * x1;
  int * x2;
  secp256k1_keypair const * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "secp256k1_keypair_xonly_pub", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(103), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_xonly_pubkey *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(103), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(104), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(104), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(71), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (secp256k1_keypair const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(71), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_keypair_xonly_pub(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_keypair_xonly_pub _cffi_d_secp256k1_keypair_xonly_pub
#endif

static int _cffi_d_secp256k1_keypair_xonly_tweak_add(secp256k1_context const * x0, secp256k1_keypair * x1, unsigned char const * x2)
{
  return secp256k1_keypair_xonly_tweak_add(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_keypair_xonly_tweak_add(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_keypair * x1;
  unsigned char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_keypair_xonly_tweak_add", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(55), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_keypair *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(55), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_keypair_xonly_tweak_add(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_keypair_xonly_tweak_add _cffi_d_secp256k1_keypair_xonly_tweak_add
#endif

static int _cffi_d_secp256k1_schnorrsig_sign32(secp256k1_context const * x0, unsigned char * x1, unsigned char const * x2, secp256k1_keypair const * x3, unsigned char const * x4)
{
  return secp256k1_schnorrsig_sign32(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_schnorrsig_sign32(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  unsigned char const * x2;
  secp256k1_keypair const * x3;
  unsigned char const * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "secp256k1_schnorrsig_sign32", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(71), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (secp256k1_keypair const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(71), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_schnorrsig_sign32(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_schnorrsig_sign32 _cffi_d_secp256k1_schnorrsig_sign32
#endif

static int _cffi_d_secp256k1_schnorrsig_sign_custom(secp256k1_context const * x0, unsigned char * x1, unsigned char const * x2, size_t x3, secp256k1_keypair const * x4, secp256k1_schnorrsig_extraparams * x5)
{
  return secp256k1_schnorrsig_sign_custom(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_schnorrsig_sign_custom(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  unsigned char const * x2;
  size_t x3;
  secp256k1_keypair const * x4;
  secp256k1_schnorrsig_extraparams * x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "secp256k1_schnorrsig_sign_custom", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(71), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (secp256k1_keypair const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(71), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(193), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (secp256k1_schnorrsig_extraparams *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(193), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_schnorrsig_sign_custom(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_schnorrsig_sign_custom _cffi_d_secp256k1_schnorrsig_sign_custom
#endif

static int _cffi_d_secp256k1_schnorrsig_verify(secp256k1_context const * x0, unsigned char const * x1, unsigned char const * x2, size_t x3, secp256k1_xonly_pubkey const * x4)
{
  return secp256k1_schnorrsig_verify(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_schnorrsig_verify(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char const * x1;
  unsigned char const * x2;
  size_t x3;
  secp256k1_xonly_pubkey const * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "secp256k1_schnorrsig_verify", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(82), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (secp256k1_xonly_pubkey const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(82), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_schnorrsig_verify(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_schnorrsig_verify _cffi_d_secp256k1_schnorrsig_verify
#endif

static secp256k1_scratch_space * _cffi_d_secp256k1_scratch_space_create(secp256k1_context const * x0, size_t x1)
{
  return secp256k1_scratch_space_create(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_scratch_space_create(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  size_t x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  secp256k1_scratch_space * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "secp256k1_scratch_space_create", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, size_t);
  if (x1 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_scratch_space_create(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(306));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_scratch_space_create _cffi_d_secp256k1_scratch_space_create
#endif

static void _cffi_d_secp256k1_scratch_space_destroy(secp256k1_context const * x0, secp256k1_scratch_space * x1)
{
  secp256k1_scratch_space_destroy(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_scratch_space_destroy(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_scratch_space * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "secp256k1_scratch_space_destroy", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(306), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_scratch_space *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(306), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { secp256k1_scratch_space_destroy(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_secp256k1_scratch_space_destroy _cffi_d_secp256k1_scratch_space_destroy
#endif

static void _cffi_d_secp256k1_selftest(void)
{
  secp256k1_selftest();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_selftest(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { secp256k1_selftest(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_secp256k1_selftest _cffi_d_secp256k1_selftest
#endif

static int _cffi_d_secp256k1_tagged_sha256(secp256k1_context const * x0, unsigned char * x1, unsigned char const * x2, size_t x3, unsigned char const * x4, size_t x5)
{
  return secp256k1_tagged_sha256(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_tagged_sha256(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  unsigned char const * x2;
  size_t x3;
  unsigned char const * x4;
  size_t x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "secp256k1_tagged_sha256", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = _cffi_to_c_int(arg5, size_t);
  if (x5 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_tagged_sha256(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_tagged_sha256 _cffi_d_secp256k1_tagged_sha256
#endif

static int _cffi_d_secp256k1_xonly_pubkey_cmp(secp256k1_context const * x0, secp256k1_xonly_pubkey const * x1, secp256k1_xonly_pubkey const * x2)
{
  return secp256k1_xonly_pubkey_cmp(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_xonly_pubkey_cmp(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_xonly_pubkey const * x1;
  secp256k1_xonly_pubkey const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_xonly_pubkey_cmp", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(82), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_xonly_pubkey const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(82), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(82), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (secp256k1_xonly_pubkey const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(82), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_xonly_pubkey_cmp(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_xonly_pubkey_cmp _cffi_d_secp256k1_xonly_pubkey_cmp
#endif

static int _cffi_d_secp256k1_xonly_pubkey_from_pubkey(secp256k1_context const * x0, secp256k1_xonly_pubkey * x1, int * x2, secp256k1_pubkey const * x3)
{
  return secp256k1_xonly_pubkey_from_pubkey(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_xonly_pubkey_from_pubkey(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_xonly_pubkey * x1;
  int * x2;
  secp256k1_pubkey const * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "secp256k1_xonly_pubkey_from_pubkey", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(103), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_xonly_pubkey *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(103), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(104), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(104), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_xonly_pubkey_from_pubkey(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_xonly_pubkey_from_pubkey _cffi_d_secp256k1_xonly_pubkey_from_pubkey
#endif

static int _cffi_d_secp256k1_xonly_pubkey_parse(secp256k1_context const * x0, secp256k1_xonly_pubkey * x1, unsigned char const * x2)
{
  return secp256k1_xonly_pubkey_parse(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_xonly_pubkey_parse(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_xonly_pubkey * x1;
  unsigned char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_xonly_pubkey_parse", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(103), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_xonly_pubkey *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(103), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_xonly_pubkey_parse(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_xonly_pubkey_parse _cffi_d_secp256k1_xonly_pubkey_parse
#endif

static int _cffi_d_secp256k1_xonly_pubkey_serialize(secp256k1_context const * x0, unsigned char * x1, secp256k1_xonly_pubkey const * x2)
{
  return secp256k1_xonly_pubkey_serialize(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_xonly_pubkey_serialize(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char * x1;
  secp256k1_xonly_pubkey const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "secp256k1_xonly_pubkey_serialize", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(125), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(125), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(82), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (secp256k1_xonly_pubkey const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(82), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_xonly_pubkey_serialize(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_xonly_pubkey_serialize _cffi_d_secp256k1_xonly_pubkey_serialize
#endif

static int _cffi_d_secp256k1_xonly_pubkey_tweak_add(secp256k1_context const * x0, secp256k1_pubkey * x1, secp256k1_xonly_pubkey const * x2, unsigned char const * x3)
{
  return secp256k1_xonly_pubkey_tweak_add(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_xonly_pubkey_tweak_add(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  secp256k1_pubkey * x1;
  secp256k1_xonly_pubkey const * x2;
  unsigned char const * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "secp256k1_xonly_pubkey_tweak_add", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(60), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (secp256k1_pubkey *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(60), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(82), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (secp256k1_xonly_pubkey const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(82), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_xonly_pubkey_tweak_add(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_xonly_pubkey_tweak_add _cffi_d_secp256k1_xonly_pubkey_tweak_add
#endif

static int _cffi_d_secp256k1_xonly_pubkey_tweak_add_check(secp256k1_context const * x0, unsigned char const * x1, int x2, secp256k1_xonly_pubkey const * x3, unsigned char const * x4)
{
  return secp256k1_xonly_pubkey_tweak_add_check(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_secp256k1_xonly_pubkey_tweak_add_check(PyObject *self, PyObject *args)
{
  secp256k1_context const * x0;
  unsigned char const * x1;
  int x2;
  secp256k1_xonly_pubkey const * x3;
  unsigned char const * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "secp256k1_xonly_pubkey_tweak_add_check", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (secp256k1_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(82), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (secp256k1_xonly_pubkey const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(82), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = secp256k1_xonly_pubkey_tweak_add_check(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_secp256k1_xonly_pubkey_tweak_add_check _cffi_d_secp256k1_xonly_pubkey_tweak_add_check
#endif

static int _cffi_const_SECP256K1_CONTEXT_DECLASSIFY(unsigned long long *o)
{
  int n = (SECP256K1_CONTEXT_DECLASSIFY) <= 0;
  *o = (unsigned long long)((SECP256K1_CONTEXT_DECLASSIFY) | 0);  /* check that SECP256K1_CONTEXT_DECLASSIFY is an integer */
  return n;
}

static int _cffi_const_SECP256K1_CONTEXT_NONE(unsigned long long *o)
{
  int n = (SECP256K1_CONTEXT_NONE) <= 0;
  *o = (unsigned long long)((SECP256K1_CONTEXT_NONE) | 0);  /* check that SECP256K1_CONTEXT_NONE is an integer */
  return n;
}

static int _cffi_const_SECP256K1_CONTEXT_SIGN(unsigned long long *o)
{
  int n = (SECP256K1_CONTEXT_SIGN) <= 0;
  *o = (unsigned long long)((SECP256K1_CONTEXT_SIGN) | 0);  /* check that SECP256K1_CONTEXT_SIGN is an integer */
  return n;
}

static int _cffi_const_SECP256K1_CONTEXT_VERIFY(unsigned long long *o)
{
  int n = (SECP256K1_CONTEXT_VERIFY) <= 0;
  *o = (unsigned long long)((SECP256K1_CONTEXT_VERIFY) | 0);  /* check that SECP256K1_CONTEXT_VERIFY is an integer */
  return n;
}

static int _cffi_const_SECP256K1_EC_COMPRESSED(unsigned long long *o)
{
  int n = (SECP256K1_EC_COMPRESSED) <= 0;
  *o = (unsigned long long)((SECP256K1_EC_COMPRESSED) | 0);  /* check that SECP256K1_EC_COMPRESSED is an integer */
  return n;
}

static int _cffi_const_SECP256K1_EC_UNCOMPRESSED(unsigned long long *o)
{
  int n = (SECP256K1_EC_UNCOMPRESSED) <= 0;
  *o = (unsigned long long)((SECP256K1_EC_UNCOMPRESSED) | 0);  /* check that SECP256K1_EC_UNCOMPRESSED is an integer */
  return n;
}

static int _cffi_const_SECP256K1_FLAGS_BIT_COMPRESSION(unsigned long long *o)
{
  int n = (SECP256K1_FLAGS_BIT_COMPRESSION) <= 0;
  *o = (unsigned long long)((SECP256K1_FLAGS_BIT_COMPRESSION) | 0);  /* check that SECP256K1_FLAGS_BIT_COMPRESSION is an integer */
  return n;
}

static int _cffi_const_SECP256K1_FLAGS_BIT_CONTEXT_DECLASSIFY(unsigned long long *o)
{
  int n = (SECP256K1_FLAGS_BIT_CONTEXT_DECLASSIFY) <= 0;
  *o = (unsigned long long)((SECP256K1_FLAGS_BIT_CONTEXT_DECLASSIFY) | 0);  /* check that SECP256K1_FLAGS_BIT_CONTEXT_DECLASSIFY is an integer */
  return n;
}

static int _cffi_const_SECP256K1_FLAGS_BIT_CONTEXT_SIGN(unsigned long long *o)
{
  int n = (SECP256K1_FLAGS_BIT_CONTEXT_SIGN) <= 0;
  *o = (unsigned long long)((SECP256K1_FLAGS_BIT_CONTEXT_SIGN) | 0);  /* check that SECP256K1_FLAGS_BIT_CONTEXT_SIGN is an integer */
  return n;
}

static int _cffi_const_SECP256K1_FLAGS_BIT_CONTEXT_VERIFY(unsigned long long *o)
{
  int n = (SECP256K1_FLAGS_BIT_CONTEXT_VERIFY) <= 0;
  *o = (unsigned long long)((SECP256K1_FLAGS_BIT_CONTEXT_VERIFY) | 0);  /* check that SECP256K1_FLAGS_BIT_CONTEXT_VERIFY is an integer */
  return n;
}

static int _cffi_const_SECP256K1_FLAGS_TYPE_COMPRESSION(unsigned long long *o)
{
  int n = (SECP256K1_FLAGS_TYPE_COMPRESSION) <= 0;
  *o = (unsigned long long)((SECP256K1_FLAGS_TYPE_COMPRESSION) | 0);  /* check that SECP256K1_FLAGS_TYPE_COMPRESSION is an integer */
  return n;
}

static int _cffi_const_SECP256K1_FLAGS_TYPE_CONTEXT(unsigned long long *o)
{
  int n = (SECP256K1_FLAGS_TYPE_CONTEXT) <= 0;
  *o = (unsigned long long)((SECP256K1_FLAGS_TYPE_CONTEXT) | 0);  /* check that SECP256K1_FLAGS_TYPE_CONTEXT is an integer */
  return n;
}

static int _cffi_const_SECP256K1_FLAGS_TYPE_MASK(unsigned long long *o)
{
  int n = (SECP256K1_FLAGS_TYPE_MASK) <= 0;
  *o = (unsigned long long)((SECP256K1_FLAGS_TYPE_MASK) | 0);  /* check that SECP256K1_FLAGS_TYPE_MASK is an integer */
  return n;
}

static int _cffi_const_SECP256K1_TAG_PUBKEY_EVEN(unsigned long long *o)
{
  int n = (SECP256K1_TAG_PUBKEY_EVEN) <= 0;
  *o = (unsigned long long)((SECP256K1_TAG_PUBKEY_EVEN) | 0);  /* check that SECP256K1_TAG_PUBKEY_EVEN is an integer */
  return n;
}

static int _cffi_const_SECP256K1_TAG_PUBKEY_ODD(unsigned long long *o)
{
  int n = (SECP256K1_TAG_PUBKEY_ODD) <= 0;
  *o = (unsigned long long)((SECP256K1_TAG_PUBKEY_ODD) | 0);  /* check that SECP256K1_TAG_PUBKEY_ODD is an integer */
  return n;
}

static int _cffi_const_SECP256K1_TAG_PUBKEY_UNCOMPRESSED(unsigned long long *o)
{
  int n = (SECP256K1_TAG_PUBKEY_UNCOMPRESSED) <= 0;
  *o = (unsigned long long)((SECP256K1_TAG_PUBKEY_UNCOMPRESSED) | 0);  /* check that SECP256K1_TAG_PUBKEY_UNCOMPRESSED is an integer */
  return n;
}

static secp256k1_context const * *_cffi_var_secp256k1_context_static(void)
{
  return &(secp256k1_context_static);
}

static const struct _cffi_global_s _cffi_globals[] = {
  { "SECP256K1_CONTEXT_DECLASSIFY", (void *)_cffi_const_SECP256K1_CONTEXT_DECLASSIFY, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "SECP256K1_CONTEXT_NONE", (void *)_cffi_const_SECP256K1_CONTEXT_NONE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "SECP256K1_CONTEXT_SIGN", (void *)_cffi_const_SECP256K1_CONTEXT_SIGN, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "SECP256K1_CONTEXT_VERIFY", (void *)_cffi_const_SECP256K1_CONTEXT_VERIFY, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "SECP256K1_EC_COMPRESSED", (void *)_cffi_const_SECP256K1_EC_COMPRESSED, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "SECP256K1_EC_UNCOMPRESSED", (void *)_cffi_const_SECP256K1_EC_UNCOMPRESSED, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "SECP256K1_FLAGS_BIT_COMPRESSION", (void *)_cffi_const_SECP256K1_FLAGS_BIT_COMPRESSION, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "SECP256K1_FLAGS_BIT_CONTEXT_DECLASSIFY", (void *)_cffi_const_SECP256K1_FLAGS_BIT_CONTEXT_DECLASSIFY, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "SECP256K1_FLAGS_BIT_CONTEXT_SIGN", (void *)_cffi_const_SECP256K1_FLAGS_BIT_CONTEXT_SIGN, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "SECP256K1_FLAGS_BIT_CONTEXT_VERIFY", (void *)_cffi_const_SECP256K1_FLAGS_BIT_CONTEXT_VERIFY, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "SECP256K1_FLAGS_TYPE_COMPRESSION", (void *)_cffi_const_SECP256K1_FLAGS_TYPE_COMPRESSION, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "SECP256K1_FLAGS_TYPE_CONTEXT", (void *)_cffi_const_SECP256K1_FLAGS_TYPE_CONTEXT, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "SECP256K1_FLAGS_TYPE_MASK", (void *)_cffi_const_SECP256K1_FLAGS_TYPE_MASK, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "SECP256K1_TAG_PUBKEY_EVEN", (void *)_cffi_const_SECP256K1_TAG_PUBKEY_EVEN, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "SECP256K1_TAG_PUBKEY_ODD", (void *)_cffi_const_SECP256K1_TAG_PUBKEY_ODD, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "SECP256K1_TAG_PUBKEY_UNCOMPRESSED", (void *)_cffi_const_SECP256K1_TAG_PUBKEY_UNCOMPRESSED, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "secp256k1_context_clone", (void *)_cffi_f_secp256k1_context_clone, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 268), (void *)_cffi_d_secp256k1_context_clone },
  { "secp256k1_context_create", (void *)_cffi_f_secp256k1_context_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 275), (void *)_cffi_d_secp256k1_context_create },
  { "secp256k1_context_destroy", (void *)_cffi_f_secp256k1_context_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 296), (void *)_cffi_d_secp256k1_context_destroy },
  { "secp256k1_context_preallocated_clone", (void *)_cffi_f_secp256k1_context_preallocated_clone, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 271), (void *)_cffi_d_secp256k1_context_preallocated_clone },
  { "secp256k1_context_preallocated_clone_size", (void *)_cffi_f_secp256k1_context_preallocated_clone_size, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 286), (void *)_cffi_d_secp256k1_context_preallocated_clone_size },
  { "secp256k1_context_preallocated_create", (void *)_cffi_f_secp256k1_context_preallocated_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 278), (void *)_cffi_d_secp256k1_context_preallocated_create },
  { "secp256k1_context_preallocated_destroy", (void *)_cffi_f_secp256k1_context_preallocated_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 296), (void *)_cffi_d_secp256k1_context_preallocated_destroy },
  { "secp256k1_context_preallocated_size", (void *)_cffi_f_secp256k1_context_preallocated_size, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 289), (void *)_cffi_d_secp256k1_context_preallocated_size },
  { "secp256k1_context_randomize", (void *)_cffi_f_secp256k1_context_randomize, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_secp256k1_context_randomize },
  { "secp256k1_context_set_error_callback", (void *)_cffi_f_secp256k1_context_set_error_callback, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 299), (void *)_cffi_d_secp256k1_context_set_error_callback },
  { "secp256k1_context_set_illegal_callback", (void *)_cffi_f_secp256k1_context_set_illegal_callback, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 299), (void *)_cffi_d_secp256k1_context_set_illegal_callback },
  { "secp256k1_context_static", (void *)_cffi_var_secp256k1_context_static, _CFFI_OP(_CFFI_OP_GLOBAL_VAR_F, 5), (void *)0 },
  { "secp256k1_ec_pubkey_cmp", (void *)_cffi_f_secp256k1_ec_pubkey_cmp, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 96), (void *)_cffi_d_secp256k1_ec_pubkey_cmp },
  { "secp256k1_ec_pubkey_combine", (void *)_cffi_f_secp256k1_ec_pubkey_combine, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 73), (void *)_cffi_d_secp256k1_ec_pubkey_combine },
  { "secp256k1_ec_pubkey_create", (void *)_cffi_f_secp256k1_ec_pubkey_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 85), (void *)_cffi_d_secp256k1_ec_pubkey_create },
  { "secp256k1_ec_pubkey_negate", (void *)_cffi_f_secp256k1_ec_pubkey_negate, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 58), (void *)_cffi_d_secp256k1_ec_pubkey_negate },
  { "secp256k1_ec_pubkey_parse", (void *)_cffi_f_secp256k1_ec_pubkey_parse, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 90), (void *)_cffi_d_secp256k1_ec_pubkey_parse },
  { "secp256k1_ec_pubkey_serialize", (void *)_cffi_f_secp256k1_ec_pubkey_serialize, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 168), (void *)_cffi_d_secp256k1_ec_pubkey_serialize },
  { "secp256k1_ec_pubkey_tweak_add", (void *)_cffi_f_secp256k1_ec_pubkey_tweak_add, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 85), (void *)_cffi_d_secp256k1_ec_pubkey_tweak_add },
  { "secp256k1_ec_pubkey_tweak_mul", (void *)_cffi_f_secp256k1_ec_pubkey_tweak_mul, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 85), (void *)_cffi_d_secp256k1_ec_pubkey_tweak_mul },
  { "secp256k1_ec_seckey_negate", (void *)_cffi_f_secp256k1_ec_seckey_negate, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 123), (void *)_cffi_d_secp256k1_ec_seckey_negate },
  { "secp256k1_ec_seckey_tweak_add", (void *)_cffi_f_secp256k1_ec_seckey_tweak_add, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 175), (void *)_cffi_d_secp256k1_ec_seckey_tweak_add },
  { "secp256k1_ec_seckey_tweak_mul", (void *)_cffi_f_secp256k1_ec_seckey_tweak_mul, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 175), (void *)_cffi_d_secp256k1_ec_seckey_tweak_mul },
  { "secp256k1_ec_seckey_verify", (void *)_cffi_f_secp256k1_ec_seckey_verify, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 219), (void *)_cffi_d_secp256k1_ec_seckey_verify },
  { "secp256k1_ecdh", (void *)_cffi_f_secp256k1_ecdh, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 149), (void *)_cffi_d_secp256k1_ecdh },
  { "secp256k1_ecdh_hash_function_default", (void *)_cffi_const_secp256k1_ecdh_hash_function_default, _CFFI_OP(_CFFI_OP_CONSTANT, 154), (void *)0 },
  { "secp256k1_ecdh_hash_function_sha256", (void *)_cffi_const_secp256k1_ecdh_hash_function_sha256, _CFFI_OP(_CFFI_OP_CONSTANT, 154), (void *)0 },
  { "secp256k1_ecdsa_recover", (void *)_cffi_f_secp256k1_ecdsa_recover, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 62), (void *)_cffi_d_secp256k1_ecdsa_recover },
  { "secp256k1_ecdsa_recoverable_signature_convert", (void *)_cffi_f_secp256k1_ecdsa_recoverable_signature_convert, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 18), (void *)_cffi_d_secp256k1_ecdsa_recoverable_signature_convert },
  { "secp256k1_ecdsa_recoverable_signature_parse_compact", (void *)_cffi_f_secp256k1_ecdsa_recoverable_signature_parse_compact, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 4), (void *)_cffi_d_secp256k1_ecdsa_recoverable_signature_parse_compact },
  { "secp256k1_ecdsa_recoverable_signature_serialize_compact", (void *)_cffi_f_secp256k1_ecdsa_recoverable_signature_serialize_compact, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 127), (void *)_cffi_d_secp256k1_ecdsa_recoverable_signature_serialize_compact },
  { "secp256k1_ecdsa_sign", (void *)_cffi_f_secp256k1_ecdsa_sign, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 39), (void *)_cffi_d_secp256k1_ecdsa_sign },
  { "secp256k1_ecdsa_sign_recoverable", (void *)_cffi_f_secp256k1_ecdsa_sign_recoverable, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 10), (void *)_cffi_d_secp256k1_ecdsa_sign_recoverable },
  { "secp256k1_ecdsa_signature_normalize", (void *)_cffi_f_secp256k1_ecdsa_signature_normalize, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 23), (void *)_cffi_d_secp256k1_ecdsa_signature_normalize },
  { "secp256k1_ecdsa_signature_parse_compact", (void *)_cffi_f_secp256k1_ecdsa_signature_parse_compact, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 28), (void *)_cffi_d_secp256k1_ecdsa_signature_parse_compact },
  { "secp256k1_ecdsa_signature_parse_der", (void *)_cffi_f_secp256k1_ecdsa_signature_parse_der, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 33), (void *)_cffi_d_secp256k1_ecdsa_signature_parse_der },
  { "secp256k1_ecdsa_signature_serialize_compact", (void *)_cffi_f_secp256k1_ecdsa_signature_serialize_compact, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 133), (void *)_cffi_d_secp256k1_ecdsa_signature_serialize_compact },
  { "secp256k1_ecdsa_signature_serialize_der", (void *)_cffi_f_secp256k1_ecdsa_signature_serialize_der, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 162), (void *)_cffi_d_secp256k1_ecdsa_signature_serialize_der },
  { "secp256k1_ecdsa_verify", (void *)_cffi_f_secp256k1_ecdsa_verify, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 47), (void *)_cffi_d_secp256k1_ecdsa_verify },
  { "secp256k1_ellswift_create", (void *)_cffi_f_secp256k1_ellswift_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 203), (void *)_cffi_d_secp256k1_ellswift_create },
  { "secp256k1_ellswift_decode", (void *)_cffi_f_secp256k1_ellswift_decode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 85), (void *)_cffi_d_secp256k1_ellswift_decode },
  { "secp256k1_ellswift_encode", (void *)_cffi_f_secp256k1_ellswift_encode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 143), (void *)_cffi_d_secp256k1_ellswift_encode },
  { "secp256k1_ellswift_xdh", (void *)_cffi_f_secp256k1_ellswift_xdh, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 209), (void *)_cffi_d_secp256k1_ellswift_xdh },
  { "secp256k1_ellswift_xdh_hash_function_bip324", (void *)_cffi_const_secp256k1_ellswift_xdh_hash_function_bip324, _CFFI_OP(_CFFI_OP_CONSTANT, 216), (void *)0 },
  { "secp256k1_ellswift_xdh_hash_function_prefix", (void *)_cffi_const_secp256k1_ellswift_xdh_hash_function_prefix, _CFFI_OP(_CFFI_OP_CONSTANT, 216), (void *)0 },
  { "secp256k1_keypair_create", (void *)_cffi_f_secp256k1_keypair_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 53), (void *)_cffi_d_secp256k1_keypair_create },
  { "secp256k1_keypair_pub", (void *)_cffi_f_secp256k1_keypair_pub, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 68), (void *)_cffi_d_secp256k1_keypair_pub },
  { "secp256k1_keypair_sec", (void *)_cffi_f_secp256k1_keypair_sec, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 138), (void *)_cffi_d_secp256k1_keypair_sec },
  { "secp256k1_keypair_xonly_pub", (void *)_cffi_f_secp256k1_keypair_xonly_pub, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 101), (void *)_cffi_d_secp256k1_keypair_xonly_pub },
  { "secp256k1_keypair_xonly_tweak_add", (void *)_cffi_f_secp256k1_keypair_xonly_tweak_add, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 53), (void *)_cffi_d_secp256k1_keypair_xonly_tweak_add },
  { "secp256k1_nonce_function_bip340", (void *)_cffi_const_secp256k1_nonce_function_bip340, _CFFI_OP(_CFFI_OP_CONSTANT, 311), (void *)0 },
  { "secp256k1_nonce_function_default", (void *)_cffi_const_secp256k1_nonce_function_default, _CFFI_OP(_CFFI_OP_CONSTANT, 15), (void *)0 },
  { "secp256k1_nonce_function_rfc6979", (void *)_cffi_const_secp256k1_nonce_function_rfc6979, _CFFI_OP(_CFFI_OP_CONSTANT, 15), (void *)0 },
  { "secp256k1_schnorrsig_sign32", (void *)_cffi_f_secp256k1_schnorrsig_sign32, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 180), (void *)_cffi_d_secp256k1_schnorrsig_sign32 },
  { "secp256k1_schnorrsig_sign_custom", (void *)_cffi_f_secp256k1_schnorrsig_sign_custom, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 187), (void *)_cffi_d_secp256k1_schnorrsig_sign_custom },
  { "secp256k1_schnorrsig_verify", (void *)_cffi_f_secp256k1_schnorrsig_verify, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 230), (void *)_cffi_d_secp256k1_schnorrsig_verify },
  { "secp256k1_scratch_space_create", (void *)_cffi_f_secp256k1_scratch_space_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 282), (void *)_cffi_d_secp256k1_scratch_space_create },
  { "secp256k1_scratch_space_destroy", (void *)_cffi_f_secp256k1_scratch_space_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 304), (void *)_cffi_d_secp256k1_scratch_space_destroy },
  { "secp256k1_selftest", (void *)_cffi_f_secp256k1_selftest, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 308), (void *)_cffi_d_secp256k1_selftest },
  { "secp256k1_tagged_sha256", (void *)_cffi_f_secp256k1_tagged_sha256, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 195), (void *)_cffi_d_secp256k1_tagged_sha256 },
  { "secp256k1_xonly_pubkey_cmp", (void *)_cffi_f_secp256k1_xonly_pubkey_cmp, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 118), (void *)_cffi_d_secp256k1_xonly_pubkey_cmp },
  { "secp256k1_xonly_pubkey_from_pubkey", (void *)_cffi_f_secp256k1_xonly_pubkey_from_pubkey, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 107), (void *)_cffi_d_secp256k1_xonly_pubkey_from_pubkey },
  { "secp256k1_xonly_pubkey_parse", (void *)_cffi_f_secp256k1_xonly_pubkey_parse, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 113), (void *)_cffi_d_secp256k1_xonly_pubkey_parse },
  { "secp256k1_xonly_pubkey_serialize", (void *)_cffi_f_secp256k1_xonly_pubkey_serialize, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 157), (void *)_cffi_d_secp256k1_xonly_pubkey_serialize },
  { "secp256k1_xonly_pubkey_tweak_add", (void *)_cffi_f_secp256k1_xonly_pubkey_tweak_add, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 79), (void *)_cffi_d_secp256k1_xonly_pubkey_tweak_add },
  { "secp256k1_xonly_pubkey_tweak_add_check", (void *)_cffi_f_secp256k1_xonly_pubkey_tweak_add_check, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 223), (void *)_cffi_d_secp256k1_xonly_pubkey_tweak_add_check },
};

static const struct _cffi_field_s _cffi_fields[] = {
  { "data", offsetof(secp256k1_ecdsa_recoverable_signature, data),
            sizeof(((secp256k1_ecdsa_recoverable_signature *)0)->data),
            _CFFI_OP(_CFFI_OP_NOOP, 325) },
  { "data", offsetof(secp256k1_ecdsa_signature, data),
            sizeof(((secp256k1_ecdsa_signature *)0)->data),
            _CFFI_OP(_CFFI_OP_NOOP, 323) },
  { "data", offsetof(secp256k1_keypair, data),
            sizeof(((secp256k1_keypair *)0)->data),
            _CFFI_OP(_CFFI_OP_NOOP, 327) },
  { "data", offsetof(secp256k1_pubkey, data),
            sizeof(((secp256k1_pubkey *)0)->data),
            _CFFI_OP(_CFFI_OP_NOOP, 323) },
  { "magic", offsetof(secp256k1_schnorrsig_extraparams, magic),
             sizeof(((secp256k1_schnorrsig_extraparams *)0)->magic),
             _CFFI_OP(_CFFI_OP_NOOP, 321) },
  { "noncefp", offsetof(secp256k1_schnorrsig_extraparams, noncefp),
               sizeof(((secp256k1_schnorrsig_extraparams *)0)->noncefp),
               _CFFI_OP(_CFFI_OP_NOOP, 311) },
  { "ndata", offsetof(secp256k1_schnorrsig_extraparams, ndata),
             sizeof(((secp256k1_schnorrsig_extraparams *)0)->ndata),
             _CFFI_OP(_CFFI_OP_NOOP, 155) },
  { "data", offsetof(secp256k1_xonly_pubkey, data),
            sizeof(((secp256k1_xonly_pubkey *)0)->data),
            _CFFI_OP(_CFFI_OP_NOOP, 323) },
};

static const struct _cffi_struct_union_s _cffi_struct_unions[] = {
  { "$secp256k1_ecdsa_recoverable_signature", 313, _CFFI_F_CHECK_FIELDS,
    sizeof(secp256k1_ecdsa_recoverable_signature), offsetof(struct _cffi_align_typedef_secp256k1_ecdsa_recoverable_signature, y), 0, 1 },
  { "$secp256k1_ecdsa_signature", 314, _CFFI_F_CHECK_FIELDS,
    sizeof(secp256k1_ecdsa_signature), offsetof(struct _cffi_align_typedef_secp256k1_ecdsa_signature, y), 1, 1 },
  { "$secp256k1_keypair", 315, _CFFI_F_CHECK_FIELDS,
    sizeof(secp256k1_keypair), offsetof(struct _cffi_align_typedef_secp256k1_keypair, y), 2, 1 },
  { "$secp256k1_pubkey", 316, _CFFI_F_CHECK_FIELDS,
    sizeof(secp256k1_pubkey), offsetof(struct _cffi_align_typedef_secp256k1_pubkey, y), 3, 1 },
  { "$secp256k1_schnorrsig_extraparams", 317, _CFFI_F_CHECK_FIELDS,
    sizeof(secp256k1_schnorrsig_extraparams), offsetof(struct _cffi_align_typedef_secp256k1_schnorrsig_extraparams, y), 4, 3 },
  { "$secp256k1_xonly_pubkey", 319, _CFFI_F_CHECK_FIELDS,
    sizeof(secp256k1_xonly_pubkey), offsetof(struct _cffi_align_typedef_secp256k1_xonly_pubkey, y), 7, 1 },
  { "secp256k1_context_struct", 312, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "secp256k1_scratch_space_struct", 318, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
};

static const struct _cffi_typename_s _cffi_typenames[] = {
  { "secp256k1_context", 312 },
  { "secp256k1_ecdh_hash_function", 154 },
  { "secp256k1_ecdsa_recoverable_signature", 313 },
  { "secp256k1_ecdsa_signature", 314 },
  { "secp256k1_ellswift_xdh_hash_function", 216 },
  { "secp256k1_keypair", 315 },
  { "secp256k1_nonce_function", 15 },
  { "secp256k1_nonce_function_hardened", 311 },
  { "secp256k1_pubkey", 316 },
  { "secp256k1_schnorrsig_extraparams", 317 },
  { "secp256k1_scratch_space", 318 },
  { "secp256k1_xonly_pubkey", 319 },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  _cffi_fields,
  _cffi_struct_unions,
  NULL,  /* no enums */
  _cffi_typenames,
  82,  /* num_globals */
  8,  /* num_struct_unions */
  0,  /* num_enums */
  12,  /* num_typenames */
  NULL,  /* no includes */
  330,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit__libsecp256k1(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit__libsecp256k1(void) { return NULL; }
#  else
     init_libsecp256k1(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit__libsecp256k1(void)
{
  return _cffi_init("_libsecp256k1", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
init_libsecp256k1(void)
{
  _cffi_init("_libsecp256k1", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
